//*****************************************************************************
// CAPT_Sandbox.h
//
//! This is the CapTIvate software library sandbox.  This can be used
//! to have version control in GIT of functions that are in development/test,
//! but not released to the full CapTIvate library yet.
//
//*****************************************************************************

#include "CAPT_Sandbox.h"
#include "rom_captivate.h"
#include "rom_map_captivate.h"

//*****************************************************************************
//
//! \var g_bEndOfConversionFlag is a global flag used to indicate to the
//! CAPT_Touch layer when a conversion has completed.  This flag is set by
//! the Captivate end-of-conversion ISR.
//
//! \var g_bConvTimerFlag
//! This flag is set by the Captivate timer ISR.
//
//! \var g_bDetectionFlag
//! This flag is set by the Captivate detection ISR.
//
//! \var g_bConvCounterFlag
//! This flag is set by the Captivate conversion counter ISR.
//
//! \var g_bMaxCountErrorFlag
//! This bool is set by the Captivate ISR when a maximum count error interrupt
//! occurs.
//
//*****************************************************************************
extern volatile bool g_bEndOfConversionFlag;
extern volatile bool g_bDetectionFlag;
extern volatile bool g_bConvTimerFlag;
extern volatile bool g_bConvCounterFlag;
extern volatile bool g_bMaxCountErrorFlag;

void CAPT_calibrateSensorToReference(tSensor *pSensor, 
    uint16_t ui16TargetDelta, uint8_t ui8RefCap)
{
	MAP_CAPT_calibrateGain(
			pSensor, 
			&g_bEndOfConversionFlag, 
			CAPT_OSC_FREQ_DEFAULT
		);
		
	CAPT_calibrateOffsetToReference(
			pSensor, 
			&g_bEndOfConversionFlag, 
			CAPT_OSC_FREQ_DEFAULT, 
			ui16TargetDelta,
			ui8RefCap
		);
	pSensor->bReCalibrateEnable = false;
}

void CAPT_calibrateOffsetToReference(tSensor *pSensor,
    volatile bool *pbEndOfConversionFlag, uint8_t ui8Freq,
    uint16_t ui16TargetDelta, uint8_t ui8RefCap)
{
    uint16_t ui16BaseCnt;
    uint16_t ui16CalCnt;
    uint16_t ui16DeltaCnt;
    uint8_t ui8Cycle;
    uint8_t ui8Element;
    uint8_t ui8CompletionMask;
    uint8_t ui8ElementMask;

    MAP_CAPT_applySensorParams(pSensor);
    MAP_CAPT_applySensorFreq(ui8Freq, pSensor);
    MAP_CAPT_bypassFSM();

    //
    // Peform calibration for each cycle within the sensor.
    //
    for (ui8Cycle=0; ui8Cycle<pSensor->ui8NrOfCycles; ui8Cycle++)
    {
        //
        // Setup completion mask, starting offset tap,
        // and set the element reseed bit
        //
        ui8CompletionMask = 0x00;
        ui8ElementMask = 0x01;
        for (ui8Element=0; ui8Element<pSensor->pCycle[ui8Cycle]->ui8NrOfElements; ui8Element++)
        {
            pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pTuning[ui8Freq].ui16OffsetTap = 1;
            pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->\
                bReseed = true;
            ui8CompletionMask |= ui8ElementMask;
            ui8ElementMask <<= 1;
        }

        //
        // Set up Captivate IO for this cycle
        //
        MAP_CAPT_setCycleIO(pSensor, (tCycle*)pSensor->pCycle[ui8Cycle]);

        //
        // Iterate through ratios to find the correct calibration
        //for all elements in this cycle.
        //
        while (ui8CompletionMask != 0)
        {
            //
            // For each element whose calibration is incomplete, update calibration settings.
            //
            ui8ElementMask = 0x01;
            for (ui8Element=0; ui8Element<pSensor->pCycle[ui8Cycle]->ui8NrOfElements; ui8Element++)
            {
                if (ui8CompletionMask & ui8ElementMask)
                {
                    //
                    // Perform a cycle conversion and save the raw results
                    //
                    MAP_CAPT_applyCycleComp(ui8Freq, (tCycle *)pSensor->pCycle[ui8Cycle]);
                    MAP_CAPT_startConversionAndWaitUntilDone(pbEndOfConversionFlag, LPM0_bits);
                    MAP_CAPT_saveCycleRawResults(
                            ui8Freq,
                            pSensor,
                            (tCycle *)pSensor->pCycle[ui8Cycle]
                        );
                    ui16BaseCnt = pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pRawCount[ui8Freq];
                    MAP_CAPT_enableRefCap(pSensor->pCycle[ui8Cycle]->pElements[ui8Element], ui8RefCap);
                    MAP_CAPT_startConversionAndWaitUntilDone(pbEndOfConversionFlag, LPM0_bits);
                    MAP_CAPT_saveCycleRawResults(
                            ui8Freq,
                            pSensor,
                            (tCycle *)pSensor->pCycle[ui8Cycle]
                        );
                    ui16CalCnt = pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pRawCount[ui8Freq];
                    MAP_CAPT_disableRefCap();
                    ui16DeltaCnt = ui16BaseCnt - ui16CalCnt;

                    if ((ui16BaseCnt == 0) || (ui16CalCnt == 0) || (ui16BaseCnt < ui16CalCnt))
                    {
                        //
                        // If arriving here, a maximum count error occured.
                        //
                        pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pTuning[ui8Freq].ui16OffsetTap--;
                        ui8CompletionMask &= ~ui8ElementMask;
                        pSensor->bCalibrationError = true;
                    }
                    else if (ui16DeltaCnt > ui16TargetDelta)
                    {
                        //
                        // If arriving here, the current offset setting resulted in the conversion exceeding the conversion count setting.
                        // The offset setting will be reduced by 1, and the calibration for this element will be
                        // marked complete.
                        //
                        pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pTuning[ui8Freq].ui16OffsetTap--;
                        ui8CompletionMask &= ~ui8ElementMask;
                    }
                    else
                    {
                        if ((pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pTuning[ui8Freq].ui16OffsetTap & 0x00FF) < 255)
                        {
                            //
                            // If arriving here, offset control still has headroom and will be increased by one notch
                            // in an attempt to reach the conversion count setting.
                            //
                            pSensor->pCycle[ui8Cycle]->pElements[ui8Element]->pTuning[ui8Freq].ui16OffsetTap++;
                        }
                        else
                        {
                            //
                            // If arriving here, the maximum offset tap has been selected
                            // and is not enough to achieve the desired conversion count setting.
                            // The calibration error flag will be set for this sensor.
                            //
                            ui8CompletionMask &= ~ui8ElementMask;
                            pSensor->bCalibrationError = true;
                        }
                    }
                } //  End of element adjustment

                //
                // Shift element index
                //
                ui8ElementMask <<= 1;

            } // End of element adjustment counting loop for this mesurment

        } // End of calibration loop for this cycle

        //
        // Done with this cycle, clear the Captivate IO
        //
        MAP_CAPT_clearCycleIO(pSensor, (tCycle*)pSensor->pCycle[ui8Cycle]);

    } // End of cycle counting loop
}
